# This file was generated automatically by Qleany's generator, edit at your own risk! 
# If you do, be careful to not overwrite it when you run the generator again.
set(PRESENTER_LIST
{%- for file in presenter_files %}
    {{ file }}
{%- endfor %}
)
